import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:intl/intl.dart' as intl;

import 'app_localizations_de.dart';
import 'app_localizations_en.dart';

/// Callers can lookup localized strings with an instance of AppLocalizations
/// returned by `AppLocalizations.of(context)`.
///
/// Applications need to include `AppLocalizations.delegate()` in their app's
/// `localizationDelegates` list, and the locales they support in the app's
/// `supportedLocales` list. For example:
///
/// ```dart
/// import 'gen_l10n/app_localizations.dart';
///
/// return MaterialApp(
///   localizationsDelegates: AppLocalizations.localizationsDelegates,
///   supportedLocales: AppLocalizations.supportedLocales,
///   home: MyApplicationHome(),
/// );
/// ```
///
/// ## Update pubspec.yaml
///
/// Please make sure to update your pubspec.yaml to include the following
/// packages:
///
/// ```yaml
/// dependencies:
///   # Internationalization support.
///   flutter_localizations:
///     sdk: flutter
///   intl: any # Use the pinned version from flutter_localizations
///
///   # Rest of dependencies
/// ```
///
/// ## iOS Applications
///
/// iOS applications define key application metadata, including supported
/// locales, in an Info.plist file that is built into the application bundle.
/// To configure the locales supported by your app, you’ll need to edit this
/// file.
///
/// First, open your project’s ios/Runner.xcworkspace Xcode workspace file.
/// Then, in the Project Navigator, open the Info.plist file under the Runner
/// project’s Runner folder.
///
/// Next, select the Information Property List item, select Add Item from the
/// Editor menu, then select Localizations from the pop-up menu.
///
/// Select and expand the newly-created Localizations item then, for each
/// locale your application supports, add a new item and select the locale
/// you wish to add from the pop-up menu in the Value field. This list should
/// be consistent with the languages listed in the AppLocalizations.supportedLocales
/// property.
abstract class AppLocalizations {
  AppLocalizations(String locale) : localeName = intl.Intl.canonicalizedLocale(locale.toString());

  final String localeName;

  static AppLocalizations of(BuildContext context) {
    return Localizations.of<AppLocalizations>(context, AppLocalizations)!;
  }

  static const LocalizationsDelegate<AppLocalizations> delegate = _AppLocalizationsDelegate();

  /// A list of this localizations delegate along with the default localizations
  /// delegates.
  ///
  /// Returns a list of localizations delegates containing this delegate along with
  /// GlobalMaterialLocalizations.delegate, GlobalCupertinoLocalizations.delegate,
  /// and GlobalWidgetsLocalizations.delegate.
  ///
  /// Additional delegates can be added by appending to this list in
  /// MaterialApp. This list does not have to be used at all if a custom list
  /// of delegates is preferred or required.
  static const List<LocalizationsDelegate<dynamic>> localizationsDelegates = <LocalizationsDelegate<dynamic>>[
    delegate,
    GlobalMaterialLocalizations.delegate,
    GlobalCupertinoLocalizations.delegate,
    GlobalWidgetsLocalizations.delegate,
  ];

  /// A list of this localizations delegate's supported locales.
  static const List<Locale> supportedLocales = <Locale>[
    Locale('de'),
    Locale('en')
  ];

  /// The Nö Finderlein (inoffical Nö Card App)
  ///
  /// In en, this message translates to:
  /// **'noefinderlein_flutter'**
  String get appTitle;

  /// No description provided for @stifte.
  ///
  /// In en, this message translates to:
  /// **'Stifte'**
  String get stifte;

  /// No description provided for @burgen_schloesser.
  ///
  /// In en, this message translates to:
  /// **'Castles and palaces'**
  String get burgen_schloesser;

  /// No description provided for @museen_ausstellungen.
  ///
  /// In en, this message translates to:
  /// **'Museums and exhibitions'**
  String get museen_ausstellungen;

  /// No description provided for @erlebnis_natur.
  ///
  /// In en, this message translates to:
  /// **'Nature and experiences'**
  String get erlebnis_natur;

  /// No description provided for @sport_und_freizeit.
  ///
  /// In en, this message translates to:
  /// **'Sport and leisure'**
  String get sport_und_freizeit;

  /// No description provided for @bergbahnen.
  ///
  /// In en, this message translates to:
  /// **'Mountain Cable/Railway'**
  String get bergbahnen;

  /// No description provided for @schifffahrt.
  ///
  /// In en, this message translates to:
  /// **'Shipping'**
  String get schifffahrt;

  /// No description provided for @lokalbahn.
  ///
  /// In en, this message translates to:
  /// **'Local railway'**
  String get lokalbahn;

  /// No description provided for @title_filter.
  ///
  /// In en, this message translates to:
  /// **'Filter settings'**
  String get title_filter;

  /// No description provided for @setFilter.
  ///
  /// In en, this message translates to:
  /// **'Activate Filter'**
  String get setFilter;

  /// No description provided for @resetFilter.
  ///
  /// In en, this message translates to:
  /// **'Reset Filter'**
  String get resetFilter;

  /// No description provided for @filteracive.
  ///
  /// In en, this message translates to:
  /// **'Filter is active'**
  String get filteracive;

  /// No description provided for @filter_day_active_text.
  ///
  /// In en, this message translates to:
  /// **'Only show locations which are open on the specified day'**
  String get filter_day_active_text;

  /// No description provided for @cancel.
  ///
  /// In en, this message translates to:
  /// **'Cancel'**
  String get cancel;

  /// No description provided for @ok.
  ///
  /// In en, this message translates to:
  /// **'OK'**
  String get ok;

  /// No description provided for @disclaimerButtonAgree.
  ///
  /// In en, this message translates to:
  /// **'Agree'**
  String get disclaimerButtonAgree;

  /// No description provided for @favorites.
  ///
  /// In en, this message translates to:
  /// **'Favorites'**
  String get favorites;

  /// No description provided for @sortby.
  ///
  /// In en, this message translates to:
  /// **'Sort by...?'**
  String get sortby;

  /// No description provided for @name.
  ///
  /// In en, this message translates to:
  /// **'Name'**
  String get name;

  /// No description provided for @dontcare.
  ///
  /// In en, this message translates to:
  /// **'don\'t care'**
  String get dontcare;

  /// No description provided for @must.
  ///
  /// In en, this message translates to:
  /// **'must'**
  String get must;

  /// No description provided for @bookletId.
  ///
  /// In en, this message translates to:
  /// **'Booklet Number'**
  String get bookletId;

  /// No description provided for @downloadingTitle.
  ///
  /// In en, this message translates to:
  /// **'Downloading data for year {year}'**
  String downloadingTitle(Object year);

  /// No description provided for @badWeather.
  ///
  /// In en, this message translates to:
  /// **'Open in bad weather'**
  String get badWeather;

  /// No description provided for @childFriendly.
  ///
  /// In en, this message translates to:
  /// **'Child friendly'**
  String get childFriendly;

  /// No description provided for @strollerFriendly.
  ///
  /// In en, this message translates to:
  /// **'Stroller friendly'**
  String get strollerFriendly;

  /// No description provided for @dogAllowed.
  ///
  /// In en, this message translates to:
  /// **'Dogs allowed'**
  String get dogAllowed;

  /// No description provided for @wheelchairFriendly.
  ///
  /// In en, this message translates to:
  /// **'Wheelchair accessible'**
  String get wheelchairFriendly;

  /// No description provided for @tavernNear.
  ///
  /// In en, this message translates to:
  /// **'Tavern near'**
  String get tavernNear;

  /// No description provided for @groupsAccepted.
  ///
  /// In en, this message translates to:
  /// **'NÖ card accepted for groups'**
  String get groupsAccepted;

  /// No description provided for @openInWinter.
  ///
  /// In en, this message translates to:
  /// **'Open in the winter'**
  String get openInWinter;

  /// No description provided for @topLocation.
  ///
  /// In en, this message translates to:
  /// **'Top location'**
  String get topLocation;

  /// No description provided for @notOpenToday.
  ///
  /// In en, this message translates to:
  /// **'Possibly not open today'**
  String get notOpenToday;

  /// No description provided for @possibleNotOpen.
  ///
  /// In en, this message translates to:
  /// **'Possibly not open on: {day}'**
  String possibleNotOpen(String day);

  /// No description provided for @yearDisplay.
  ///
  /// In en, this message translates to:
  /// **'Year {year1}/{year2}'**
  String yearDisplay(String year1, String year2);

  /// No description provided for @map.
  ///
  /// In en, this message translates to:
  /// **'Map'**
  String get map;

  /// No description provided for @readon.
  ///
  /// In en, this message translates to:
  /// **'read on'**
  String get readon;

  /// No description provided for @search.
  ///
  /// In en, this message translates to:
  /// **'Search'**
  String get search;

  /// No description provided for @drawerAllLocations.
  ///
  /// In en, this message translates to:
  /// **'All Locations'**
  String get drawerAllLocations;

  /// No description provided for @drawerProvinces.
  ///
  /// In en, this message translates to:
  /// **'Provinces'**
  String get drawerProvinces;

  /// No description provided for @drawerFavorits.
  ///
  /// In en, this message translates to:
  /// **'Favorites'**
  String get drawerFavorits;

  /// No description provided for @drawerVisited.
  ///
  /// In en, this message translates to:
  /// **'Visited Locations'**
  String get drawerVisited;

  /// No description provided for @drawerNear.
  ///
  /// In en, this message translates to:
  /// **'Near Locations'**
  String get drawerNear;

  /// No description provided for @drawerSettings.
  ///
  /// In en, this message translates to:
  /// **'Settings'**
  String get drawerSettings;

  /// No description provided for @settingsDownload.
  ///
  /// In en, this message translates to:
  /// **'Restart app and start download again'**
  String get settingsDownload;

  /// No description provided for @drawerCards.
  ///
  /// In en, this message translates to:
  /// **'My Cards'**
  String get drawerCards;

  /// No description provided for @drawerAbout.
  ///
  /// In en, this message translates to:
  /// **'About'**
  String get drawerAbout;

  /// No description provided for @settingsTitle.
  ///
  /// In en, this message translates to:
  /// **'Settings'**
  String get settingsTitle;

  /// No description provided for @noVisited.
  ///
  /// In en, this message translates to:
  /// **'It seems you haven\'t visited any locations yet...\n\nOr maybe you have forgotten to mark a location as visited ✔️ on the detail page?'**
  String get noVisited;

  /// No description provided for @visitedRemoveTitle.
  ///
  /// In en, this message translates to:
  /// **'Delete entry?'**
  String get visitedRemoveTitle;

  /// No description provided for @includedInCard.
  ///
  /// In en, this message translates to:
  /// **'Included in the CARD'**
  String get includedInCard;

  /// No description provided for @savings.
  ///
  /// In en, this message translates to:
  /// **'Savings'**
  String get savings;

  /// No description provided for @adultCard.
  ///
  /// In en, this message translates to:
  /// **'Adult-CARD'**
  String get adultCard;

  /// No description provided for @childCard.
  ///
  /// In en, this message translates to:
  /// **'Child-CARD'**
  String get childCard;

  /// No description provided for @noCard.
  ///
  /// In en, this message translates to:
  /// **'There was no saved Card found, please add a Card first.'**
  String get noCard;

  /// No description provided for @mycardsTitle.
  ///
  /// In en, this message translates to:
  /// **'My Cards'**
  String get mycardsTitle;

  /// No description provided for @downloadError.
  ///
  /// In en, this message translates to:
  /// **'There was an error downloading. Maybe there is no internet-connection?\n\nThe error was: {error}'**
  String downloadError(String error);

  /// No description provided for @visitedScreenTitle.
  ///
  /// In en, this message translates to:
  /// **'Visited'**
  String get visitedScreenTitle;

  /// No description provided for @systemTheme.
  ///
  /// In en, this message translates to:
  /// **'System Theme'**
  String get systemTheme;

  /// No description provided for @lightTheme.
  ///
  /// In en, this message translates to:
  /// **'Light Theme'**
  String get lightTheme;

  /// No description provided for @darkTheme.
  ///
  /// In en, this message translates to:
  /// **'Dark Theme'**
  String get darkTheme;

  /// No description provided for @regionenScreenTitle.
  ///
  /// In en, this message translates to:
  /// **'Provinces'**
  String get regionenScreenTitle;

  /// No description provided for @nearScreenTitle.
  ///
  /// In en, this message translates to:
  /// **'Near locations'**
  String get nearScreenTitle;

  /// No description provided for @positionNotFound.
  ///
  /// In en, this message translates to:
  /// **'The current position could not be found. Try to activiate the location settings for this app and globaly on this device.\n\n{log}'**
  String positionNotFound(String log);

  /// No description provided for @textClear.
  ///
  /// In en, this message translates to:
  /// **'Clear'**
  String get textClear;

  /// No description provided for @openLocationSettings.
  ///
  /// In en, this message translates to:
  /// **'Open Location Settings'**
  String get openLocationSettings;

  /// No description provided for @openAppSettings.
  ///
  /// In en, this message translates to:
  /// **'Open App Settings'**
  String get openAppSettings;

  /// No description provided for @reqTempFullAcc.
  ///
  /// In en, this message translates to:
  /// **'Request Temporary Full Accuracy'**
  String get reqTempFullAcc;

  /// No description provided for @getLocAcc.
  ///
  /// In en, this message translates to:
  /// **'Get Location Accuracy'**
  String get getLocAcc;

  /// No description provided for @saveVisitedMessage.
  ///
  /// In en, this message translates to:
  /// **'Saved amount € {amount} on Date {date}'**
  String saveVisitedMessage(String amount, String date);

  /// No description provided for @detailCall.
  ///
  /// In en, this message translates to:
  /// **'CALL'**
  String get detailCall;

  /// No description provided for @detailMail.
  ///
  /// In en, this message translates to:
  /// **'MAIL'**
  String get detailMail;

  /// No description provided for @detailWeb.
  ///
  /// In en, this message translates to:
  /// **'WEBPAGE'**
  String get detailWeb;

  /// No description provided for @detailNav.
  ///
  /// In en, this message translates to:
  /// **'NAVI'**
  String get detailNav;

  /// No description provided for @detailFav.
  ///
  /// In en, this message translates to:
  /// **'FAV'**
  String get detailFav;

  /// No description provided for @disclaimer.
  ///
  /// In en, this message translates to:
  /// **'The developer of this app does not guarantee the completeness and the correctness of the showed data. If you continue to use this app, you confirm that you are aware of this.'**
  String get disclaimer;

  /// No description provided for @drawerFinderleinWebsite.
  ///
  /// In en, this message translates to:
  /// **'finderlein.at'**
  String get drawerFinderleinWebsite;

  /// No description provided for @drawerGithub.
  ///
  /// In en, this message translates to:
  /// **'Contribute on Github'**
  String get drawerGithub;

  /// No description provided for @drawerLicenses.
  ///
  /// In en, this message translates to:
  /// **'Licenses'**
  String get drawerLicenses;

  /// No description provided for @closeSearch.
  ///
  /// In en, this message translates to:
  /// **'Close Search'**
  String get closeSearch;

  /// No description provided for @locationListOnTapHint.
  ///
  /// In en, this message translates to:
  /// **'View Details'**
  String get locationListOnTapHint;

  /// No description provided for @ttDownloadData.
  ///
  /// In en, this message translates to:
  /// **'Download new data'**
  String get ttDownloadData;

  /// No description provided for @ttChangeSorting.
  ///
  /// In en, this message translates to:
  /// **'Change sort Order'**
  String get ttChangeSorting;

  /// No description provided for @ttFilterList.
  ///
  /// In en, this message translates to:
  /// **'Filter the list'**
  String get ttFilterList;

  /// No description provided for @ttShowLocationsOnMap.
  ///
  /// In en, this message translates to:
  /// **'Show locations on a map'**
  String get ttShowLocationsOnMap;

  /// No description provided for @checkInWhichDay.
  ///
  /// In en, this message translates to:
  /// **'CheckIn for which day?'**
  String get checkInWhichDay;

  /// No description provided for @addAdult.
  ///
  /// In en, this message translates to:
  /// **'Add adult'**
  String get addAdult;

  /// No description provided for @addChild.
  ///
  /// In en, this message translates to:
  /// **'Add child'**
  String get addChild;

  /// No description provided for @saveCheckin.
  ///
  /// In en, this message translates to:
  /// **'Save'**
  String get saveCheckin;

  /// No description provided for @checkinSetTo.
  ///
  /// In en, this message translates to:
  /// **'Set to € 0'**
  String get checkinSetTo;
}

class _AppLocalizationsDelegate extends LocalizationsDelegate<AppLocalizations> {
  const _AppLocalizationsDelegate();

  @override
  Future<AppLocalizations> load(Locale locale) {
    return SynchronousFuture<AppLocalizations>(lookupAppLocalizations(locale));
  }

  @override
  bool isSupported(Locale locale) => <String>['de', 'en'].contains(locale.languageCode);

  @override
  bool shouldReload(_AppLocalizationsDelegate old) => false;
}

AppLocalizations lookupAppLocalizations(Locale locale) {


  // Lookup logic when only language code is specified.
  switch (locale.languageCode) {
    case 'de': return AppLocalizationsDe();
    case 'en': return AppLocalizationsEn();
  }

  throw FlutterError(
    'AppLocalizations.delegate failed to load unsupported locale "$locale". This is likely '
    'an issue with the localizations generation tool. Please file an issue '
    'on GitHub with a reproducible sample app and the gen-l10n configuration '
    'that was used.'
  );
}
